__author__ = 'michel'
# -*- coding: utf-8 -*-

import datetime, os, urllib, sys
import decimal

from sqlalchemy import Boolean, Column, Date, DateTime, Float, Integer, String, Text, Time, ForeignKey
from libraries.database.database import db


class Model:
    def store(self, commit=False):
        db.session.add(self)
        if commit:
            self.commit()

    def commit(self):
        db.session.commit()

#    def dump_datetime(self, value):
#        """Deserialize datetime object into string form for JSON processing."""
#        if value is None:
#            return None
#        return [value.strftime("%Y-%m-%d"), value.strftime("%H:%M:%S")]

    def alchemy_encoder(obj):
        """JSON encoder function for SQLAlchemy special classes."""
        if isinstance(obj, datetime.date):
            return obj.isoformat()
        elif isinstance(obj, decimal.Decimal):
            return float(obj)

    def _asdict(self):
        return self.__dict__


# ------------------------------------------------------------------------------------------ #
class ScheduleModel(Model):
    def get_length(self):
        sec1 = int(datetime.datetime.strptime(self.start[0:16].replace(" ", "T"), "%Y-%m-%dT%H:%M").strftime("%s"))
        sec2 = int(datetime.datetime.strptime(self.end[0:16].replace(" ", "T"), "%Y-%m-%dT%H:%M").strftime("%s"))
        len = sec2 - sec1
        return len


# ------------------------------------------------------------------------------------------ #
class ScheduleEntryModel(Model):
    # ------------------------------------------------------------------------------------------ #
    @staticmethod
    def upcoming(datefrom=datetime.datetime.now()):
        upcomingtracks = ScheduleEntry.query.filter(ScheduleEntry.start > datefrom).all()
        return upcomingtracks

    # ------------------------------------------------------------------------------------------ #
    @staticmethod
    def select_all():
        return ScheduleEntry.query.filter().all()

    @staticmethod
    def select_one(playlist_id, entry_num):
        return ScheduleEntry.query.filter(ScheduleEntry.playlist_id == playlist_id, ScheduleEntry.entry_num == entry_num).first()

    def __str__(self):
        return "ScheduleEntry starts @ " + ScheduleEntry.entry_start + " and ends @ " + ScheduleEntry.entry_end + " and plays " + ScheduleEntry.source


# ------------------------------------------------------------------------------------------ #
class Schedule(db.Model, ScheduleModel):
    """
    One specific Schedule for a show on a timeslot
    """
    __tablename__ = 'schedule'
    schedule_id = Column(Integer, primary_key=True, autoincrement=False)
    show_id = Column(Integer)

    schedule_start = Column(DateTime, nullable=False)
    schedule_end = Column(DateTime, nullable=False)
    show_name = Column(String(256))
    show_hosts = Column(String(256))
    rtr_category = Column(String(256))
    comment = Column(String(512))
    languages = Column(String(256))
    type = Column(String(256))
    category = Column(String(256))
    topic = Column(String(256))
    musicfocus = Column(String(256))

    is_repetition = Column(Boolean())

    playlist_id = Column(Integer)
    timeslot_fallback_id = Column(Integer)
    show_fallback_id = Column(Integer)
    station_fallback_id = Column(Integer)

    @property
    def serialize(self):
        """Return object data in easily serializeable format"""
        return {
            'schedule_id': self.schedule_id,
            'show_id': self.show_id,
            'schedule_start': self.dump_datetime(self.schedule_start),
            'schedule_end': self.dump_datetime(self.schedule_end),
            'show_name': self.show_name,
            'show_hosts': self.show_hosts,
            'is_repetition': self.is_repetition,
            'fallback_playlist': self.fallback_playlist,
            'fallback_pool': self.fallback_pool,
            'station_fallback_pool': self.station_fallback_pool
        }


# ------------------------------------------------------------------------------------------ #
class ScheduleEntry(db.Model, ScheduleEntryModel):
    """
    One schedule can have multiple entries
    """
    __tablename__ = 'schedule_entry'

    playlist_id = Column(Integer, primary_key=True, nullable=False, autoincrement=False)
    entry_num = Column(Integer, primary_key=True, nullable=False, autoincrement=False)
    schedule_id = Column(Integer, ForeignKey("schedule.schedule_id"))
    entry_start = Column(DateTime, nullable=False)
    entry_end = Column(DateTime, nullable=False)
    source = Column(String(256))
    artist = Column(String(256))
    track = Column(String(256))
    albumname = Column(String(256))
    genre = Column(String(256))
    tracknum = Column(String(256))
    cdnum = Column(String(256))
    year = Column(String(5)) # well should be int, but can also be ""

#    def __init__(self, playlist_id, entry_num, schedule_id, entry_start, entry_end, source, artist, track, albumname, genre, tracknum, cdnum, year):
#        self.playlist_id = playlist_id
#        self.entry_num = entry_num
#        self.schedule_id = schedule_id
#        self.entry_start = entry_start
#        self.entry_end = entry_end
#        self.source = source
#        self.artist = artist
#        self.track = track
#        self.albumname = albumname
#        self.genre = genre
#        self.tracknum = tracknum
#        self.cdnum = cdnum
#        self.year = year

    @property
    def serialize(self):
        """Return object data in easily serializeable format"""
        return {
            'schedule_id': self.schedule_id,
            'entry_start': self.dump_datetime(self.entry_start),
            'entry_end': self.dump_datetime(self.entry_end),
            'source': self.source
        }

#class Trackservice(db.Model, Model):
#    """
#    Trackservice and Logging
#    """
#    __tablename__ = 'trackservice'
#    id = Column(Integer, primary_key=True)
#    show_id = Column(Integer, ForeignKey("timeslot.id"))


def recreateDB():
    print("Recreating Database...")
    db.drop_all()
    print("all dropped. creating...")
    db.create_all()
    print("all created. commiting...")
    db.session.commit()
    print("Database recreated!")
    sys.exit(0)

#recreateDB()
